//#include "metalShaderTypes.h"

typedef struct
{
	float speed;
} fxVars;

constant float PI = 3.14159265;

// Radial blur samples. More is always better, but there's frame rate to consider.
constant float SAMPLES = 18.;


// 2x1 hash. Used to jitter the samples.
float hash( vec2 p ){ return fract(sin(dot(p, vec2(41, 289)))*45758.5453); }


// Light offset.
//
// I realized, after a while, that determining the correct light position doesn't help, since
// radial blur doesn't really look right unless its focus point is within the screen boundaries,
// whereas the light is often out of frame. Therefore, I decided to go for something that at
// least gives the feel of following the light. In this case, I normalized the light position
// and rotated it in unison with the camera rotation. Hacky, for sure, but who's checking? :)
vec3 lOff(float fsTime)
{
    
    vec2 u = sin(vec2(1.57, 0) - fsTime/2.);
    mat2 a = mat2(u.x,u.y, -u.y, u.x);
    
    vec3 l = normalize(vec3(1.5, 1., -0.5));
    l.xz = a * l.xz;
    l.xy = a * l.xy;
    
    return l;
    
}



vertex fxVertexOut vertexFunc(uint vertexID [[ vertex_id ]]
							  ,const device fxShaderVerts* in[[ buffer(0) ]]
							  ,texture2d<half> inputTex0 [[ texture(0) ]]
							  ,texture2d<half> inputTexMask [[ texture(1) ]]
							  ,constant fxGeneralUniforms& u[[ buffer(1) ]]
							  ,constant fxVars& vars[[ buffer(2)]])
{
    fxVertexOut out;

	out.normPos=in[vertexID].pos.xy*vec2(.5,-.5f);//+vec2(.5,.5);
	out.position=in[vertexID].pos;
	out.texSize=float2(inputTex0.get_width(),inputTex0.get_height());
	out.tex1=in[vertexID].tex1;
	out.texMask=in[vertexID].texMask;
    return out;
}

fragment float4 fragmentFunc(fxVertexOut in [[stage_in]]
							 ,texture2d<half> inputTex [[ texture(0) ]]
							 ,texture2d<half> inputTexMask [[ texture(1) ]]
							 ,constant fxGeneralUniforms& u[[ buffer(0) ]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
    float fsTime=u.localTime*vars.speed;

    float cLength = length(in.normPos.xy);
    
    vec2 uv=in.tex1+(in.normPos.xy/cLength)*cos(cLength*12.0-fsTime*4.0)*0.03;

	vec4 texel=tex(inputTex,fsTexture);
	float4 outColor=tex(inputTex,uv);
	
	float blendValue=u.blendValue;
	
#ifdef USE_MASK
    blendValue*=tex(inputTexMask,in.texMask).a;
#endif
    outColor.rgb=mix(texel.rgb,outColor.rgb,blendValue);
	return outColor;
}
