//#include "metalShaderTypes.h"

typedef struct
{
	float radius;
} fxVars;

vertex fxVertexOut vertexFunc(uint vertexID [[ vertex_id ]]
							  ,const device fxShaderVerts* in[[ buffer(0) ]]
							  ,texture2d<half> inputTex0 [[ texture(0) ]]
							  ,texture2d<half> inputTexMask [[ texture(1) ]]
							  ,constant fxGeneralUniforms& u[[ buffer(1) ]]
							  ,constant fxVars& vars[[ buffer(2)]])
{
    fxVertexOut out;

	out.normPos=in[vertexID].pos.xy*vec2(.5,-.5f);//+vec2(.5,.5);
	out.position=in[vertexID].pos;
	out.texSize=float2(inputTex0.get_width(),inputTex0.get_height());
	out.tex1=in[vertexID].tex1;
	out.texMask=in[vertexID].texMask;

    return out;
}

fragment float4 fragmentFunc(fxVertexOut in [[stage_in]]
							 ,texture2d<half> inputTex [[ texture(0) ]]
							 ,texture2d<half> inputTexMask [[ texture(1) ]]
							 ,constant fxGeneralUniforms& u[[ buffer(0) ]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
    float fsOuterRadius=vars.radius/100.0;
    float fsInnerRadius=fsOuterRadius*.5;

    vec4 texel = tex(inputTex, fsTexture);
    float dist=distance(in.normPos,vec2(0.0,0.0));
    float darken=clamp((fsOuterRadius-dist)/(fsOuterRadius-fsInnerRadius),0.0,1.0);
	float4 outColor;
	outColor.rgb=texel.rgb*darken;
    outColor.a=texel.a;
	
	float blendValue=u.blendValue;
#ifdef USE_MASK
	blendValue*=tex(in.texMask,inputTexMask).a;
#endif
	outColor.rgb=mix(texel.rgb,outColor.rgb,blendValue);
	return outColor;
}
